"""Airzone Cloud API Aidoo device."""

from __future__ import annotations

from typing import Any

from .common import parse_str
from .const import (
    API_AZ_AIDOO,
    API_AZ_AIDOO_PRO,
    API_MODE,
    API_NAME,
    API_POWER,
    API_SETPOINT,
    API_SP_AIR_COOL,
    API_SP_AIR_HEAT,
    API_SPEED_CONF,
    API_TYPE,
    API_VALUE,
    AZD_MODEL,
)
from .hvac import HVAC


class Aidoo(HVAC):
    """Airzone Cloud Aidoo device."""

    def __init__(self, inst_id: str, ws_id: str, device_data: dict[str, Any]):
        """Airzone Cloud Aidoo device init."""
        super().__init__(inst_id, ws_id, device_data)

        self.aidoo_type: str | None = None

        device_name = parse_str(device_data.get(API_NAME))
        if device_name is not None:
            self.name = device_name
        else:
            self.name = f"Aidoo {ws_id}"

        device_type = parse_str(device_data.get(API_TYPE))
        if device_type is not None:
            if device_type == API_AZ_AIDOO:
                self.aidoo_type = "Aidoo"
            elif device_type == API_AZ_AIDOO_PRO:
                self.aidoo_type = "Aidoo Pro"

    def data(self) -> dict[str, Any]:
        """Return Aidoo device data."""
        data = super().data()

        aidoo_type = self.get_aidoo_type()
        if aidoo_type is not None:
            data[AZD_MODEL] = aidoo_type

        return data

    def get_aidoo_type(self) -> str | None:
        """Return Aidoo type."""
        return self.aidoo_type

    def set_param(self, param: str, data: dict[str, Any]) -> None:
        """Update device parameter from API request."""
        if param == API_MODE:
            self.set_mode(data[API_VALUE])
        elif param == API_POWER:
            self.set_power(data[API_VALUE])
        elif param == API_SETPOINT:
            self.set_setpoint(data[API_VALUE])
        elif param == API_SP_AIR_COOL:
            self.set_setpoint_cool(data[API_VALUE])
        elif param == API_SP_AIR_HEAT:
            self.set_setpoint_heat(data[API_VALUE])
        elif param == API_SPEED_CONF:
            self.set_speed(data[API_VALUE])



"""Airzone Cloud API HVAC device."""

from __future__ import annotations

from typing import Any

from .common import (
    AirQualityMode,
    OperationAction,
    OperationMode,
    SpeedType,
    parse_bool,
    parse_float,
    parse_int,
    parse_str,
)
from .const import (
    API_ACTIVE,
    API_AIR_ACTIVE,
    API_AQ_ACTIVE,
    API_AQ_MODE_CONF,
    API_AQ_MODE_VALUES,
    API_CELSIUS,
    API_CONSUMPTION_UE,
    API_DEFAULT_TEMP_STEP,
    API_DISCH_COMP_TEMP_UE,
    API_DOUBLE_SET_POINT,
    API_EXCH_HEAT_TEMP_IU,
    API_EXCH_HEAT_TEMP_UE,
    API_EXT_TEMP,
    API_HUMIDITY,
    API_LOCAL_TEMP,
    API_PE_UE,
    API_POWER,
    API_RAD_ACTIVE,
    API_RADIO_BATTERY_PERCENT,
    API_RADIO_COVERAGE_PERCENT,
    API_RANGE_MAX_AIR,
    API_RANGE_MIN_AIR,
    API_RANGE_SP_MAX_AUTO_AIR,
    API_RANGE_SP_MAX_COOL_AIR,
    API_RANGE_SP_MAX_DRY_AIR,
    API_RANGE_SP_MAX_EMERHEAT_AIR,
    API_RANGE_SP_MAX_HOT_AIR,
    API_RANGE_SP_MAX_STOP_AIR,
    API_RANGE_SP_MAX_VENT_AIR,
    API_RANGE_SP_MIN_AUTO_AIR,
    API_RANGE_SP_MIN_COOL_AIR,
    API_RANGE_SP_MIN_DRY_AIR,
    API_RANGE_SP_MIN_EMERHEAT_AIR,
    API_RANGE_SP_MIN_HOT_AIR,
    API_RANGE_SP_MIN_STOP_AIR,
    API_RANGE_SP_MIN_VENT_AIR,
    API_RETURN_TEMP,
    API_SP_AIR_COOL,
    API_SP_AIR_DRY,
    API_SP_AIR_HEAT,
    API_SP_AIR_STOP,
    API_SP_AIR_VENT,
    API_SPEED_CONF,
    API_SPEED_TYPE,
    API_SPEED_VALUES,
    API_STEP,
    API_THERMOSTAT_FW,
    API_THERMOSTAT_TYPE,
    API_WORK_TEMP,
    AZD_ACTION,
    AZD_ACTIVE,
    AZD_AQ_ACTIVE,
    AZD_AQ_MODE_CONF,
    AZD_AQ_MODE_VALUES,
    AZD_DOUBLE_SET_POINT,
    AZD_HUMIDITY,
    AZD_INDOOR_EXCHANGER_TEMP,
    AZD_INDOOR_RETURN_TEMP,
    AZD_INDOOR_WORK_TEMP,
    AZD_OUTDOOR_DISCHARGE_PRESS,
    AZD_OUTDOOR_DISCHARGE_TEMP,
    AZD_OUTDOOR_ELECTRIC_CURRENT,
    AZD_OUTDOOR_EXCHANGER_TEMP,
    AZD_OUTDOOR_TEMP,
    AZD_POWER,
    AZD_SPEED,
    AZD_SPEED_TYPE,
    AZD_SPEEDS,
    AZD_TEMP,
    AZD_TEMP_SET,
    AZD_TEMP_SET_AUTO_AIR,
    AZD_TEMP_SET_COOL_AIR,
    AZD_TEMP_SET_DRY_AIR,
    AZD_TEMP_SET_HOT_AIR,
    AZD_TEMP_SET_MAX,
    AZD_TEMP_SET_MAX_AUTO_AIR,
    AZD_TEMP_SET_MAX_COOL_AIR,
    AZD_TEMP_SET_MAX_DRY_AIR,
    AZD_TEMP_SET_MAX_EMERHEAT_AIR,
    AZD_TEMP_SET_MAX_HOT_AIR,
    AZD_TEMP_SET_MAX_STOP_AIR,
    AZD_TEMP_SET_MAX_VENT_AIR,
    AZD_TEMP_SET_MIN,
    AZD_TEMP_SET_MIN_AUTO_AIR,
    AZD_TEMP_SET_MIN_COOL_AIR,
    AZD_TEMP_SET_MIN_DRY_AIR,
    AZD_TEMP_SET_MIN_EMERHEAT_AIR,
    AZD_TEMP_SET_MIN_HOT_AIR,
    AZD_TEMP_SET_MIN_STOP_AIR,
    AZD_TEMP_SET_MIN_VENT_AIR,
    AZD_TEMP_SET_STOP_AIR,
    AZD_TEMP_SET_VENT_AIR,
    AZD_TEMP_STEP,
    AZD_THERMOSTAT_BATTERY,
    AZD_THERMOSTAT_COVERAGE,
    AZD_THERMOSTAT_FW,
    AZD_THERMOSTAT_MODEL,
)
from .device import Device
from .entity import EntityUpdate, UpdateType


class HVAC(Device):
    """Airzone Cloud HVAC device."""

    def __init__(self, inst_id: str, ws_id: str, device_data: dict[str, Any]):
        """Airzone Cloud HVAC device init."""
        super().__init__(inst_id, ws_id, device_data)

        self.active: bool | None = None
        self.air_demand: bool | None = None
        self.aq_active: bool | None = None
        self.aq_mode_conf: AirQualityMode | None = None
        self.aq_mode_values: list[AirQualityMode] | None = None
        self.double_set_point: bool | None = None
        self.floor_demand: bool | None = None
        self.humidity: int | None = None
        self.indoor_exchanger_temp: float | None = None
        self.indoor_return_temp: float | None = None
        self.indoor_work_temp: float | None = None
        self.name: str = "HVAC"
        self.outdoor_electric_current: float | None = None
        self.outdoor_discharge_press: float | None = None
        self.outdoor_discharge_temp: float | None = None
        self.outdoor_exchanger_temp: float | None = None
        self.outdoor_temp: float | None = None
        self.power: bool | None = None
        self.speed: int | None = None
        self.speeds: dict[int, int] = {}
        self.speed_type: SpeedType | None = None
        self.temp_set_max: float | None = None
        self.temp_set_max_auto_air: float | None = None
        self.temp_set_max_cool_air: float | None = None
        self.temp_set_max_dry_air: float | None = None
        self.temp_set_max_emerheat_air: float | None = None
        self.temp_set_max_hot_air: float | None = None
        self.temp_set_max_stop_air: float | None = None
        self.temp_set_max_vent_air: float | None = None
        self.temp_set_min: float | None = None
        self.temp_set_min_auto_air: float | None = None
        self.temp_set_min_cool_air: float | None = None
        self.temp_set_min_dry_air: float | None = None
        self.temp_set_min_emerheat_air: float | None = None
        self.temp_set_min_hot_air: float | None = None
        self.temp_set_min_stop_air: float | None = None
        self.temp_set_min_vent_air: float | None = None
        self.temp_set_auto_air: float | None = None
        self.temp_set_cool_air: float | None = None
        self.temp_set_dry_air: float | None = None
        self.temp_set_hot_air: float | None = None
        self.temp_set_stop_air: float | None = None
        self.temp_set_vent_air: float | None = None
        self.temp: float | None = None
        self.temp_step: float | None = None
        self.thermostat_battery: int | None = None
        self.thermostat_coverage: int | None = None
        self.thermostat_fw: str | None = None
        self.thermostat_type: str | None = None

    def data(self) -> dict[str, Any]:
        """Return HVAC device data."""
        data = super().data()

        data[AZD_ACTION] = self.get_action()
        data[AZD_ACTIVE] = self.get_active()
        data[AZD_DOUBLE_SET_POINT] = self.get_double_set_point()
        data[AZD_POWER] = self.get_power()
        data[AZD_TEMP] = self.get_temperature()
        data[AZD_TEMP_STEP] = self.get_temp_step()

        aq_active = self.get_aq_active()
        if aq_active is not None:
            data[AZD_AQ_ACTIVE] = aq_active

        aq_mode_conf = self.get_aq_mode_conf()
        if aq_mode_conf is not None:
            data[AZD_AQ_MODE_CONF] = aq_mode_conf

        aq_mode_values = self.get_aq_mode_values()
        if aq_mode_values is not None:
            data[AZD_AQ_MODE_VALUES] = aq_mode_values

        humidity = self.get_humidity()
        if humidity is not None:
            data[AZD_HUMIDITY] = humidity

        indoor_exchanger_temperature = self.get_indoor_exchanger_temperature()
        if indoor_exchanger_temperature is not None:
            data[AZD_INDOOR_EXCHANGER_TEMP] = indoor_exchanger_temperature

        indoor_return_temperature = self.get_indoor_return_temperature()
        if indoor_return_temperature is not None:
            data[AZD_INDOOR_RETURN_TEMP] = indoor_return_temperature

        indoor_work_temperature = self.get_indoor_work_temperature()
        if indoor_work_temperature is not None:
            data[AZD_INDOOR_WORK_TEMP] = indoor_work_temperature

        outdoor_discharge_press = self.get_outdoor_discharge_pressure()
        if outdoor_discharge_press is not None:
            data[AZD_OUTDOOR_DISCHARGE_PRESS] = outdoor_discharge_press

        outdoor_discharge_temp = self.get_outdoor_discharge_temperature()
        if outdoor_discharge_temp is not None:
            data[AZD_OUTDOOR_DISCHARGE_TEMP] = outdoor_discharge_temp

        outdoor_electric_current = self.get_outdoor_electric_current()
        if outdoor_electric_current is not None:
            data[AZD_OUTDOOR_ELECTRIC_CURRENT] = outdoor_electric_current

        outdoor_exchanger_temperature = self.get_outdoor_exchanger_temperature()
        if outdoor_exchanger_temperature is not None:
            data[AZD_OUTDOOR_EXCHANGER_TEMP] = outdoor_exchanger_temperature

        outdoor_temperature = self.get_outdoor_temperature()
        if outdoor_temperature is not None:
            data[AZD_OUTDOOR_TEMP] = outdoor_temperature

        speed = self.get_speed()
        if speed is not None:
            data[AZD_SPEED] = speed

        speeds = self.get_speeds()
        if speeds is not None:
            data[AZD_SPEEDS] = speeds

        speed_type = self.get_speed_type()
        if speed_type is not None:
            data[AZD_SPEED_TYPE] = speed_type

        temp_set_max = self.get_temp_set_max()
        if temp_set_max is not None:
            data[AZD_TEMP_SET_MAX] = temp_set_max
        temp_set_max_auto_air = self.get_temp_set_max_auto_air()
        if temp_set_max_auto_air is not None:
            data[AZD_TEMP_SET_MAX_AUTO_AIR] = temp_set_max_auto_air
        temp_set_max_cool_air = self.get_temp_set_max_cool_air()
        if temp_set_max_cool_air is not None:
            data[AZD_TEMP_SET_MAX_COOL_AIR] = temp_set_max_cool_air
        temp_set_max_dry_air = self.get_temp_set_max_dry_air()
        if temp_set_max_dry_air is not None:
            data[AZD_TEMP_SET_MAX_DRY_AIR] = temp_set_max_dry_air
        temp_set_max_emerheat_air = self.get_temp_set_max_emerheat_air()
        if temp_set_max_emerheat_air is not None:
            data[AZD_TEMP_SET_MAX_EMERHEAT_AIR] = temp_set_max_emerheat_air
        temp_set_max_hot_air = self.get_temp_set_max_hot_air()
        if temp_set_max_hot_air is not None:
            data[AZD_TEMP_SET_MAX_HOT_AIR] = temp_set_max_hot_air
        temp_set_max_stop_air = self.get_temp_set_max_stop_air()
        if temp_set_max_stop_air is not None:
            data[AZD_TEMP_SET_MAX_STOP_AIR] = temp_set_max_stop_air
        temp_set_max_vent_air = self.get_temp_set_max_vent_air()
        if temp_set_max_vent_air is not None:
            data[AZD_TEMP_SET_MAX_VENT_AIR] = temp_set_max_vent_air

        temp_set_min = self.get_temp_set_min()
        if temp_set_min is not None:
            data[AZD_TEMP_SET_MIN] = temp_set_min
        temp_set_min_auto_air = self.get_temp_set_min_auto_air()
        if temp_set_min_auto_air is not None:
            data[AZD_TEMP_SET_MIN_AUTO_AIR] = temp_set_min_auto_air
        temp_set_min_cool_air = self.get_temp_set_min_cool_air()
        if temp_set_min_cool_air is not None:
            data[AZD_TEMP_SET_MIN_COOL_AIR] = temp_set_min_cool_air
        temp_set_min_dry_air = self.get_temp_set_min_dry_air()
        if temp_set_min_dry_air is not None:
            data[AZD_TEMP_SET_MIN_DRY_AIR] = temp_set_min_dry_air
        temp_set_min_emerheat_air = self.get_temp_set_min_emerheat_air()
        if temp_set_min_emerheat_air is not None:
            data[AZD_TEMP_SET_MIN_EMERHEAT_AIR] = temp_set_min_emerheat_air
        temp_set_min_hot_air = self.get_temp_set_min_hot_air()
        if temp_set_min_hot_air is not None:
            data[AZD_TEMP_SET_MIN_HOT_AIR] = temp_set_min_hot_air
        temp_set_min_stop_air = self.get_temp_set_min_stop_air()
        if temp_set_min_stop_air is not None:
            data[AZD_TEMP_SET_MIN_STOP_AIR] = temp_set_min_stop_air
        temp_set_min_vent_air = self.get_temp_set_min_vent_air()
        if temp_set_min_vent_air is not None:
            data[AZD_TEMP_SET_MIN_VENT_AIR] = temp_set_min_vent_air

        temp_set = self.get_temp_set()
        if temp_set is not None:
            data[AZD_TEMP_SET] = temp_set
        temp_set_auto_air = self.get_temp_set_auto_air()
        if temp_set_auto_air is not None:
            data[AZD_TEMP_SET_AUTO_AIR] = temp_set_auto_air
        temp_set_cool_air = self.get_temp_set_cool_air()
        if temp_set_cool_air is not None:
            data[AZD_TEMP_SET_COOL_AIR] = temp_set_cool_air
        temp_set_dry_air = self.get_temp_set_dry_air()
        if temp_set_dry_air is not None:
            data[AZD_TEMP_SET_DRY_AIR] = temp_set_dry_air
        temp_set_hot_air = self.get_temp_set_hot_air()
        if temp_set_hot_air is not None:
            data[AZD_TEMP_SET_HOT_AIR] = temp_set_hot_air
        temp_set_stop_air = self.get_temp_set_stop_air()
        if temp_set_stop_air is not None:
            data[AZD_TEMP_SET_STOP_AIR] = temp_set_stop_air
        temp_set_vent_air = self.get_temp_set_vent_air()
        if temp_set_vent_air is not None:
            data[AZD_TEMP_SET_VENT_AIR] = temp_set_vent_air

        thermostat_battery = self.get_thermostat_battery()
        if thermostat_battery is not None:
            data[AZD_THERMOSTAT_BATTERY] = thermostat_battery

        thermostat_coverage = self.get_thermostat_coverage()
        if thermostat_coverage is not None:
            data[AZD_THERMOSTAT_COVERAGE] = thermostat_coverage

        thermostat_firmware = self.get_thermostat_fw()
        if thermostat_firmware is not None:
            data[AZD_THERMOSTAT_FW] = thermostat_firmware

        thermostat_model = self.get_thermostat_type()
        if thermostat_model is not None:
            data[AZD_THERMOSTAT_MODEL] = thermostat_model

        return data

    def get_action(self) -> OperationAction:
        """Return HVAC action."""
        if self.get_power():
            if self.get_active():
                mode = self.get_mode() or OperationMode.STOP
                if mode.is_cool():
                    action = OperationAction.COOLING
                elif mode.is_heat():
                    action = OperationAction.HEATING
                elif mode.is_vent():
                    action = OperationAction.FAN
                elif mode.is_dry():
                    action = OperationAction.DRYING
                elif mode.is_auto():
                    action = self.get_action_auto()
                else:
                    action = OperationAction.OFF
            else:
                action = OperationAction.IDLE
        else:
            action = OperationAction.OFF
        return action

    def get_action_auto(self) -> OperationAction:
        """Return current Auto action."""
        mode = self.get_mode_auto()
        if mode is not None:
            if mode.is_cool():
                action = OperationAction.COOLING
            elif mode.is_heat():
                action = OperationAction.HEATING
            elif mode.is_vent():
                action = OperationAction.FAN
            elif mode.is_dry():
                action = OperationAction.DRYING
            else:
                action = OperationAction.IDLE
        else:
            action = OperationAction.IDLE
        return action

    def get_active(self) -> bool | None:
        """Return HVAC device active status."""
        return self.active

    def get_air_demand(self) -> bool | None:
        """Return HVAC device air_demand status."""
        return self.air_demand

    def get_aq_active(self) -> bool | None:
        """Return HVAC device Air Quality active status."""
        return self.aq_active

    def get_aq_mode_conf(self) -> AirQualityMode | None:
        """Return HVAC device Air Quality mode conf."""
        return self.aq_mode_conf

    def get_aq_mode_values(self) -> list[AirQualityMode] | None:
        """Return HVAC device Air Quality mode values."""
        if self.aq_mode_values is not None and len(self.aq_mode_values) > 0:
            return self.aq_mode_values
        return None

    def get_double_set_point(self) -> bool:
        """Return HVAC double set point."""
        if self.double_set_point is not None:
            return self.double_set_point
        return False

    def get_floor_demand(self) -> bool | None:
        """Return HVAC device floor demand status."""
        return self.floor_demand

    def get_indoor_return_temperature(self) -> float | None:
        """Return HVAC indoor return temperature."""
        if self.indoor_return_temp is not None:
            return round(self.indoor_return_temp, 1)
        return None

    def get_indoor_exchanger_temperature(self) -> float | None:
        """Return HVAC indoor heat exchanger temperature."""
        if self.indoor_exchanger_temp is not None:
            return round(self.indoor_exchanger_temp, 1)
        return None

    def get_indoor_work_temperature(self) -> float | None:
        """Return HVAC indoor work temperature."""
        if self.indoor_work_temp is not None:
            return round(self.indoor_work_temp, 1)
        return None

    def get_humidity(self) -> int | None:
        """Return HVAC device humidity."""
        return self.humidity

    def get_outdoor_electric_current(self) -> float | None:
        """Return HVAC outdoor electric current."""
        return self.outdoor_electric_current

    def get_outdoor_discharge_pressure(self) -> float | None:
        """Return HVAC outdoor compressor discharge pressure."""
        if self.outdoor_discharge_press is not None:
            return self.outdoor_discharge_press * 1000
        return None

    def get_outdoor_discharge_temperature(self) -> float | None:
        """Return HVAC outdoor compressor discharge temperature."""
        if self.outdoor_discharge_temp is not None:
            return round(self.outdoor_discharge_temp, 1)
        return None

    def get_outdoor_exchanger_temperature(self) -> float | None:
        """Return HVAC outdoor heat exchanger temperature."""
        if self.outdoor_exchanger_temp is not None:
            return round(self.outdoor_exchanger_temp, 1)
        return None

    def get_outdoor_temperature(self) -> float | None:
        """Return HVAC outdoor temperature."""
        if self.outdoor_temp is not None:
            return round(self.outdoor_temp, 1)
        return None

    def get_power(self) -> bool | None:
        """Return HVAC device power."""
        return self.power

    def get_speed(self) -> int | None:
        """Return HVAC speed."""
        return self.speed

    def get_speeds(self) -> dict[int, int] | None:
        """Return HVAC speeds."""
        if len(self.speeds) > 0:
            return self.speeds
        return None

    def get_speed_type(self) -> SpeedType | None:
        """Return HVAC speed type."""
        return self.speed_type

    def set_speed(self, speed: int) -> None:
        """Set HVAC speed."""
        if self.speed is not None:
            self.speed = speed

    def get_temperature(self) -> float | None:
        """Return HVAC device temperature."""
        if self.temp is not None:
            return round(self.temp, 1)
        return None

    def get_temp_set(self) -> float | None:
        """Return HVAC device temperature setpoint."""
        temp_set: float | None = None
        mode = self.get_mode()
        if mode is not None:
            if mode.is_auto():
                temp_set = self.get_temp_set_auto_air()
            elif mode.is_cool():
                temp_set = self.get_temp_set_cool_air()
            elif mode.is_dry():
                temp_set = self.get_temp_set_dry_air()
            elif mode.is_heat():
                temp_set = self.get_temp_set_hot_air()
            elif mode.is_stop():
                temp_set = self.get_temp_set_stop_air()
            elif mode.is_vent():
                temp_set = self.get_temp_set_vent_air()
        return temp_set

    def get_temp_set_auto_air(self) -> float | None:
        """Return HVAC device setpoint for Auto Air."""
        if self.temp_set_auto_air is not None:
            return round(self.temp_set_auto_air, 1)
        return None

    def get_temp_set_cool_air(self) -> float | None:
        """Return HVAC device setpoint for Cool Air."""
        if self.temp_set_cool_air is not None:
            return round(self.temp_set_cool_air, 1)
        return None

    def get_temp_set_dry_air(self) -> float | None:
        """Return HVAC device setpoint for Dry Air."""
        if self.temp_set_dry_air is not None:
            return round(self.temp_set_dry_air, 1)
        return None

    def get_temp_set_hot_air(self) -> float | None:
        """Return HVAC device setpoint for Heat Air."""
        if self.temp_set_hot_air is not None:
            return round(self.temp_set_hot_air, 1)
        return None

    def get_temp_set_stop_air(self) -> float | None:
        """Return HVAC device setpoint for Stop Air."""
        if self.temp_set_stop_air is not None:
            return round(self.temp_set_stop_air, 1)
        return None

    def get_temp_set_vent_air(self) -> float | None:
        """Return HVAC device setpoint for Ventilation Air."""
        if self.temp_set_vent_air is not None:
            return round(self.temp_set_vent_air, 1)
        return None

    def get_temp_set_max(self) -> float | None:
        """Return HVAC device max setpoint."""
        max_temp: float | None = None
        temps = [
            self.temp_set_max,
            self.get_temp_set_max_auto_air(),
            self.get_temp_set_max_cool_air(),
            self.get_temp_set_max_dry_air(),
            self.get_temp_set_max_emerheat_air(),
            self.get_temp_set_max_hot_air(),
            self.get_temp_set_max_stop_air(),
            self.get_temp_set_max_vent_air(),
        ]
        for temp in temps:
            if temp is not None:
                if max_temp is None or temp > max_temp:
                    max_temp = temp
        return max_temp

    def get_temp_set_max_auto_air(self) -> float | None:
        """Return HVAC device max setpoint for Auto Air."""
        if self.temp_set_max_auto_air is not None:
            return round(self.temp_set_max_auto_air, 1)
        return None

    def get_temp_set_max_cool_air(self) -> float | None:
        """Return HVAC device max setpoint for Cool Air."""
        if self.temp_set_max_cool_air is not None:
            return round(self.temp_set_max_cool_air, 1)
        return None

    def get_temp_set_max_dry_air(self) -> float | None:
        """Return HVAC device max setpoint for Dry Air."""
        if self.temp_set_max_dry_air is not None:
            return round(self.temp_set_max_dry_air, 1)
        return None

    def get_temp_set_max_emerheat_air(self) -> float | None:
        """Return HVAC device max setpoint for Emergency Heat Air."""
        if self.temp_set_max_emerheat_air is not None:
            return round(self.temp_set_max_emerheat_air, 1)
        return None

    def get_temp_set_max_hot_air(self) -> float | None:
        """Return HVAC device max setpoint for Heat Air."""
        if self.temp_set_max_hot_air is not None:
            return round(self.temp_set_max_hot_air, 1)
        return None

    def get_temp_set_max_stop_air(self) -> float | None:
        """Return HVAC device max setpoint for Stop Air."""
        if self.temp_set_max_stop_air is not None:
            return round(self.temp_set_max_stop_air, 1)
        return None

    def get_temp_set_max_vent_air(self) -> float | None:
        """Return HVAC device max setpoint for Ventilation Air."""
        if self.temp_set_max_vent_air is not None:
            return round(self.temp_set_max_vent_air, 1)
        return None

    def get_temp_set_min(self) -> float | None:
        """Return HVAC device min setpoint."""
        min_temp: float | None = None
        temps = [
            self.temp_set_min,
            self.get_temp_set_min_auto_air(),
            self.get_temp_set_min_cool_air(),
            self.get_temp_set_min_dry_air(),
            self.get_temp_set_min_emerheat_air(),
            self.get_temp_set_min_hot_air(),
            self.get_temp_set_min_stop_air(),
            self.get_temp_set_min_vent_air(),
        ]
        for temp in temps:
            if temp is not None:
                if min_temp is None or temp < min_temp:
                    min_temp = temp
        return min_temp

    def get_temp_set_min_auto_air(self) -> float | None:
        """Return HVAC device min setpoint for Auto Air."""
        if self.temp_set_min_auto_air:
            return round(self.temp_set_min_auto_air, 1)
        return None

    def get_temp_set_min_cool_air(self) -> float | None:
        """Return HVAC device min setpoint for Cool Air."""
        if self.temp_set_min_cool_air:
            return round(self.temp_set_min_cool_air, 1)
        return None

    def get_temp_set_min_dry_air(self) -> float | None:
        """Return HVAC device min setpoint for Dry Air."""
        if self.temp_set_min_dry_air is not None:
            return round(self.temp_set_min_dry_air, 1)
        return None

    def get_temp_set_min_emerheat_air(self) -> float | None:
        """Return HVAC device min setpoint for Emergency Heat Air."""
        if self.temp_set_min_emerheat_air is not None:
            return round(self.temp_set_min_emerheat_air, 1)
        return None

    def get_temp_set_min_hot_air(self) -> float | None:
        """Return HVAC device min setpoint for Hot Air."""
        if self.temp_set_min_hot_air is not None:
            return round(self.temp_set_min_hot_air, 1)
        return None

    def get_temp_set_min_stop_air(self) -> float | None:
        """Return HVAC device min setpoint for Stop Air."""
        if self.temp_set_min_stop_air is not None:
            return round(self.temp_set_min_stop_air, 1)
        return None

    def get_temp_set_min_vent_air(self) -> float | None:
        """Return HVAC device min setpoint for Ventilation Air."""
        if self.temp_set_min_vent_air is not None:
            return round(self.temp_set_min_vent_air, 1)
        return None

    def get_temp_step(self) -> float | None:
        """Return HVAC device temperature step."""
        if self.temp_step is not None:
            return round(self.temp_step, 1)
        return API_DEFAULT_TEMP_STEP

    def get_thermostat_battery(self) -> int | None:
        """Return Thermostat battery."""
        return self.thermostat_battery

    def get_thermostat_coverage(self) -> int | None:
        """Return Thermostat coverage."""
        return self.thermostat_coverage

    def get_thermostat_fw(self) -> str | None:
        """Return Thermostat firmware."""
        return self.thermostat_fw

    def get_thermostat_type(self) -> str | None:
        """Return Thermostat type."""
        return self.thermostat_type

    def set_aq_mode(self, aq_mode: AirQualityMode) -> None:
        """Set HVAC Air Quality mode."""
        self.aq_mode_conf = aq_mode

    def set_power(self, power: bool) -> None:
        """Set HVAC power."""
        self.power = power

    def set_setpoint(self, setpoint: float) -> None:
        """Set HVAC setpoint."""
        if self.temp_set_auto_air is not None:
            self.temp_set_auto_air = setpoint
        if self.temp_set_cool_air is not None:
            self.temp_set_cool_air = setpoint
        if self.temp_set_dry_air is not None:
            self.temp_set_dry_air = setpoint
        if self.temp_set_hot_air is not None:
            self.temp_set_hot_air = setpoint
        if self.temp_set_stop_air is not None:
            self.temp_set_stop_air = setpoint
        if self.temp_set_vent_air is not None:
            self.temp_set_vent_air = setpoint

    def set_setpoint_cool(self, setpoint: float) -> None:
        """Set HVAC cool setpoint."""
        if self.temp_set_cool_air is not None:
            self.temp_set_cool_air = setpoint

    def set_setpoint_heat(self, setpoint: float) -> None:
        """Set HVAC heat setpoint."""
        if self.temp_set_hot_air is not None:
            self.temp_set_hot_air = setpoint

    def update_data(self, update: EntityUpdate) -> None:
        """Update HVAC device data."""
        super().update_data(update)

        data = update.get_data()

        if API_ACTIVE in data:
            active = parse_bool(data.get(API_ACTIVE))
            if active is not None:
                self.active = active
            else:
                # API sends active as null instead of False
                self.active = False
        else:
            if update.get_type() != UpdateType.WS_PARTIAL:
                self.active = None

        if API_AIR_ACTIVE in data:
            air_demand = parse_bool(data.get(API_AIR_ACTIVE))
            if air_demand is not None:
                self.air_demand = air_demand
            else:
                # API sends API_AIR_ACTIVE as null instead of False
                self.air_demand = False
        else:
            if update.get_type() != UpdateType.WS_PARTIAL:
                self.air_demand = None

        if API_RAD_ACTIVE in data:
            floor_demand = parse_bool(data.get(API_RAD_ACTIVE))
            if floor_demand is not None:
                self.floor_demand = floor_demand
            else:
                # API sends API_RAD_ACTIVE as null instead of False
                self.floor_demand = False
        else:
            if update.get_type() != UpdateType.WS_PARTIAL:
                self.floor_demand = None

        aq_active = parse_bool(data.get(API_AQ_ACTIVE))
        if aq_active is not None:
            self.aq_active = aq_active

        aq_mode_conf = data.get(API_AQ_MODE_CONF)
        if aq_mode_conf is not None:
            self.aq_mode_conf = AirQualityMode(aq_mode_conf)

        aq_mode_values = data.get(API_AQ_MODE_VALUES)
        if aq_mode_values is not None:
            self.aq_mode_values = []
            for aq_mode_value in aq_mode_values:
                self.aq_mode_values += [AirQualityMode(aq_mode_value)]

        double_set_point = parse_bool(data.get(API_DOUBLE_SET_POINT))
        if double_set_point is not None:
            self.double_set_point = double_set_point

        humidity = parse_int(data.get(API_HUMIDITY))
        if humidity is not None:
            self.humidity = humidity

        local_temp = parse_float(data.get(API_LOCAL_TEMP, {}).get(API_CELSIUS))
        if local_temp is not None:
            self.temp = local_temp

        indoor_return_temp = parse_float(data.get(API_RETURN_TEMP, {}).get(API_CELSIUS))
        if indoor_return_temp is not None:
            self.indoor_return_temp = indoor_return_temp

        indoor_exchanger_temp = parse_float(
            data.get(API_EXCH_HEAT_TEMP_IU, {}).get(API_CELSIUS)
        )
        if indoor_exchanger_temp is not None:
            self.indoor_exchanger_temp = indoor_exchanger_temp

        indoor_work_temp = parse_float(data.get(API_WORK_TEMP, {}).get(API_CELSIUS))
        if indoor_work_temp is not None:
            self.indoor_work_temp = indoor_work_temp

        outdoor_electric_current = parse_float(data.get(API_CONSUMPTION_UE))
        if outdoor_electric_current is not None:
            self.outdoor_electric_current = outdoor_electric_current

        outdoor_discharge_press = parse_float(data.get(API_PE_UE))
        if outdoor_discharge_press is not None:
            self.outdoor_discharge_press = outdoor_discharge_press

        outdoor_discharge_temp = parse_float(
            data.get(API_DISCH_COMP_TEMP_UE, {}).get(API_CELSIUS)
        )
        if outdoor_discharge_temp is not None:
            self.outdoor_discharge_temp = outdoor_discharge_temp

        outdoor_exchanger_temp = parse_float(
            data.get(API_EXCH_HEAT_TEMP_UE, {}).get(API_CELSIUS)
        )
        if outdoor_exchanger_temp is not None:
            self.outdoor_exchanger_temp = outdoor_exchanger_temp

        outdoor_temp = parse_float(data.get(API_EXT_TEMP, {}).get(API_CELSIUS))
        if outdoor_temp is not None:
            self.outdoor_temp = outdoor_temp

        power = parse_bool(data.get(API_POWER))
        if power is not None:
            self.power = power

        range_max_air = parse_float(data.get(API_RANGE_MAX_AIR, {}).get(API_CELSIUS))
        if range_max_air is not None:
            self.temp_set_max = range_max_air
        range_sp_max_auto_air = parse_float(
            data.get(API_RANGE_SP_MAX_AUTO_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_auto_air is not None:
            self.temp_set_max_auto_air = range_sp_max_auto_air
        range_sp_max_cool_air = parse_float(
            data.get(API_RANGE_SP_MAX_COOL_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_cool_air is not None:
            self.temp_set_max_cool_air = range_sp_max_cool_air
        range_sp_max_dry_air = parse_float(
            data.get(API_RANGE_SP_MAX_DRY_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_dry_air is not None:
            self.temp_set_max_dry_air = range_sp_max_dry_air
        range_sp_max_emerheat_air = parse_float(
            data.get(API_RANGE_SP_MAX_EMERHEAT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_emerheat_air is not None:
            self.temp_set_max_emerheat_air = range_sp_max_emerheat_air
        range_sp_max_hot_air = parse_float(
            data.get(API_RANGE_SP_MAX_HOT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_hot_air is not None:
            self.temp_set_max_hot_air = range_sp_max_hot_air
        range_sp_max_stop_air = parse_float(
            data.get(API_RANGE_SP_MAX_STOP_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_stop_air is not None:
            self.temp_set_max_stop_air = range_sp_max_stop_air
        range_sp_max_vent_air = parse_float(
            data.get(API_RANGE_SP_MAX_VENT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_max_vent_air is not None:
            self.temp_set_max_vent_air = range_sp_max_vent_air

        range_min_air = parse_float(data.get(API_RANGE_MIN_AIR, {}).get(API_CELSIUS))
        if range_min_air is not None:
            self.temp_set_min = range_min_air
        range_sp_min_auto_air = parse_float(
            data.get(API_RANGE_SP_MIN_AUTO_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_auto_air is not None:
            self.temp_set_min_auto_air = range_sp_min_auto_air
        range_sp_min_cool_air = parse_float(
            data.get(API_RANGE_SP_MIN_COOL_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_cool_air is not None:
            self.temp_set_min_cool_air = range_sp_min_cool_air
        range_sp_min_dry_air = parse_float(
            data.get(API_RANGE_SP_MIN_DRY_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_dry_air is not None:
            self.temp_set_min_dry_air = range_sp_min_dry_air
        range_sp_min_emerheat_air = parse_float(
            data.get(API_RANGE_SP_MIN_EMERHEAT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_emerheat_air is not None:
            self.temp_set_min_emerheat_air = range_sp_min_emerheat_air
        range_sp_min_hot_air = parse_float(
            data.get(API_RANGE_SP_MIN_HOT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_hot_air is not None:
            self.temp_set_min_hot_air = range_sp_min_hot_air
        range_sp_min_stop_air = parse_float(
            data.get(API_RANGE_SP_MIN_STOP_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_stop_air is not None:
            self.temp_set_min_stop_air = range_sp_min_stop_air
        range_sp_min_vent_air = parse_float(
            data.get(API_RANGE_SP_MIN_VENT_AIR, {}).get(API_CELSIUS)
        )
        if range_sp_min_vent_air is not None:
            self.temp_set_min_vent_air = range_sp_min_vent_air

        speed = parse_int(data.get(API_SPEED_CONF))
        if speed is not None:
            self.speed = speed

        speed_type = data.get(API_SPEED_TYPE)
        if speed_type is not None:
            self.speed_type = SpeedType(speed_type)

        speeds_values: list[int] | None = data.get(API_SPEED_VALUES)
        if speeds_values is not None:
            speeds: dict[int, int] = {}

            if 0 in speeds_values:
                speeds[0] = 0

            speed_count = 1
            speeds_values.sort()
            for speed_value in speeds_values:
                if speed_value > 0:
                    speeds[speed_count] = int(speed_value)
                    speed_count += 1

            self.speeds = speeds

        sp_air_cool = parse_float(data.get(API_SP_AIR_COOL, {}).get(API_CELSIUS))
        if sp_air_cool is not None:
            self.temp_set_cool_air = sp_air_cool
        sp_air_dry = parse_float(data.get(API_SP_AIR_DRY, {}).get(API_CELSIUS))
        if sp_air_dry is not None:
            self.temp_set_dry_air = sp_air_dry
        sp_air_heat = parse_float(data.get(API_SP_AIR_HEAT, {}).get(API_CELSIUS))
        if sp_air_heat is not None:
            self.temp_set_hot_air = sp_air_heat
        sp_air_stop = parse_float(data.get(API_SP_AIR_STOP, {}).get(API_CELSIUS))
        if sp_air_stop is not None:
            self.temp_set_stop_air = sp_air_stop
        sp_air_vent = parse_float(data.get(API_SP_AIR_VENT, {}).get(API_CELSIUS))
        if sp_air_vent is not None:
            self.temp_set_vent_air = sp_air_vent

        step = parse_float(data.get(API_STEP, {}).get(API_CELSIUS))
        if step is not None:
            self.temp_step = step

        thermostat_battery = parse_int(data.get(API_RADIO_BATTERY_PERCENT))
        if thermostat_battery is not None:
            self.thermostat_battery = thermostat_battery

        thermostat_coverage = parse_int(data.get(API_RADIO_COVERAGE_PERCENT))
        if thermostat_coverage is not None:
            self.thermostat_coverage = thermostat_coverage

        thermostat_fw = parse_str(data.get(API_THERMOSTAT_FW))
        if thermostat_fw is not None:
            self.thermostat_fw = thermostat_fw

        thermostat_type = parse_str(data.get(API_THERMOSTAT_TYPE))
        if thermostat_type is not None:
            self.thermostat_type = thermostat_type



"""Airzone Cloud API System."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .common import parse_str
from .const import (
    API_MODE,
    API_SYSTEM_FW,
    API_SYSTEM_NUMBER,
    API_SYSTEM_TYPE,
    API_VALUE,
    AZD_FIRMWARE,
    AZD_MODEL,
    AZD_SYSTEM,
)
from .device import Device
from .entity import EntityUpdate

if TYPE_CHECKING:
    from .zone import Zone


class System(Device):
    """Airzone Cloud System device."""

    def __init__(self, inst_id: str, ws_id: str, device_data: dict[str, Any]):
        """Airzone Cloud System device init."""
        super().__init__(inst_id, ws_id, device_data)

        self.system_fw: str | None = None
        self.system_type: str | None = None
        self.zones: dict[str, Zone] = {}

        sub_data = self.sub_data(device_data)
        self.system_number = int(sub_data[API_SYSTEM_NUMBER])

        self.name = f"System {self.system_number}"

    def data(self) -> dict[str, Any]:
        """Return System device data."""
        data = super().data()

        data[AZD_SYSTEM] = self.get_system_num()

        system_fw = self.get_system_fw()
        if system_fw is not None:
            data[AZD_FIRMWARE] = system_fw

        system_type = self.get_system_type()
        if system_type is not None:
            data[AZD_MODEL] = system_type

        return data

    def add_zone(self, zone: Zone) -> None:
        """Add System zone."""
        zone_id = zone.get_id()
        if zone_id not in self.zones:
            self.zones[zone_id] = zone

    def get_system_fw(self) -> str | None:
        """Return System firmware."""
        return self.system_fw

    def get_system_num(self) -> int:
        """Return System number."""
        return self.system_number

    def get_system_type(self) -> str | None:
        """Return System type."""
        return self.system_type

    def set_param(self, param: str, data: dict[str, Any]) -> None:
        """Update System parameter from API request."""

        if param == API_MODE:
            self.set_mode(data[API_VALUE])
            for zone in self.zones.values():
                zone.set_mode(data[API_VALUE])

    def update_data(self, update: EntityUpdate) -> None:
        """Update System data."""
        super().update_data(update)

        data = update.get_data()

        system_fw = parse_str(data.get(API_SYSTEM_FW))
        if system_fw is not None:
            self.system_fw = system_fw

        system_type = parse_str(data.get(API_SYSTEM_TYPE))
        if system_type is not None:
            self.system_type = system_type



"""Airzone Cloud API constants."""

from datetime import timedelta
from typing import Final

AIRZONE_SERVER: Final[str] = "m.airzonecloud.com"

API_ACCESS_TYPE: Final[str] = "access_type"
API_ACTIVE: Final[str] = "active"
API_AIR_ACTIVE: Final[str] = "air_active"
API_AUTH_LOGIN: Final[str] = "auth/login"
API_AUTH_REFRESH_TOKEN: Final[str] = "auth/refreshToken"
API_AUTO_MODE: Final[str] = "auto_mode"
API_AQ_ACTIVE: Final[str] = "aq_active"
API_AQ_MODE_CONF: Final[str] = "aq_mode_conf"
API_AQ_MODE_VALUES: Final[str] = "aq_mode_values"
API_AQ_PM_1: Final[str] = "aqpm1_0"
API_AQ_PM_2P5: Final[str] = "aqpm2_5"
API_AQ_PM_10: Final[str] = "aqpm10"
API_AQ_PRESENT: Final[str] = "aq_present"
API_AQ_QUALITY: Final[str] = "aq_quality"
API_CELSIUS: Final[str] = "celsius"
API_CITY: Final[str] = "city"
API_CONFIG: Final[str] = "config"
API_CONNECTION_DATE: Final[str] = "connection_date"
API_CONSUMPTION_UE: Final[str] = "consumption_ue"
API_DEVICE_ID: Final[str] = "device_id"
API_DEVICE_TYPE: Final[str] = "device_type"
API_DEVICES: Final[str] = "devices"
API_DISCH_COMP_TEMP_UE: Final[str] = "disch_comp_temp_ue"
API_DISCONNECTION_DATE: Final[str] = "disconnection_date"
API_DOUBLE_SET_POINT: Final[str] = "double_sp"
API_ECO_CONF: Final[str] = "eco_conf"
API_EMAIL: Final[str] = "email"
API_ERRORS: Final[str] = "errors"
API_EXCH_HEAT_TEMP_IU: Final[str] = "exch_heat_temp_iu"
API_EXCH_HEAT_TEMP_UE: Final[str] = "exch_heat_temp_ue"
API_FAH: Final[str] = "fah"
API_GROUP: Final[str] = "group"
API_GROUP_ID: Final[str] = "group_id"
API_GROUPS: Final[str] = "groups"
API_HUMIDITY: Final[str] = "humidity"
API_INSTALLATION_ID: Final[str] = "installation_id"
API_INSTALLATIONS: Final[str] = "installations"
API_IS_CONNECTED: Final[str] = "isConnected"
API_LOCAL_TEMP: Final[str] = "local_temp"
API_LOCATION_ID: Final[str] = "location_id"
API_MACHINE_READY: Final[str] = "machineready"
API_META: Final[str] = "meta"
API_MODE: Final[str] = "mode"
API_MODE_AVAIL: Final[str] = "mode_available"
API_NAME: Final[str] = "name"
API_OLD_ID: Final[str] = "_id"
API_EXT_TEMP: Final[str] = "ext_temp"
API_OPTS: Final[str] = "opts"
API_PARAM: Final[str] = "param"
API_PARAMS: Final[str] = "params"
API_PASSWORD: Final[str] = "password"
API_PE_UE: Final[str] = "pe_ue"
API_PIN: Final[str] = "pin"
API_POWER: Final[str] = "power"
API_POWERFUL_MODE: Final[str] = "powerful_mode"
API_RAD_ACTIVE: Final[str] = "rad_active"
API_RADIO_BATTERY_PERCENT: Final[str] = "radio_batt_percent"
API_RADIO_COVERAGE_PERCENT: Final[str] = "radio_coverage_percent"
API_RANGE_MAX_AIR: Final[str] = "range_air_max"
API_RANGE_MIN_AIR: Final[str] = "range_air_min"
API_RANGE_SP_MAX_ACS: Final[str] = "range_sp_acs_max"
API_RANGE_SP_MAX_AUTO_AIR: Final[str] = "range_sp_auto_air_max"
API_RANGE_SP_MAX_COOL_AIR: Final[str] = "range_sp_cool_air_max"
API_RANGE_SP_MAX_DRY_AIR: Final[str] = "range_sp_dry_air_max"
API_RANGE_SP_MAX_EMERHEAT_AIR: Final[str] = "range_sp_emerheat_air_max"
API_RANGE_SP_MAX_HOT_AIR: Final[str] = "range_sp_hot_air_max"
API_RANGE_SP_MAX_STOP_AIR: Final[str] = "range_sp_stop_air_max"
API_RANGE_SP_MAX_VENT_AIR: Final[str] = "range_sp_vent_air_max"
API_RANGE_SP_MIN_ACS: Final[str] = "range_sp_acs_min"
API_RANGE_SP_MIN_AUTO_AIR: Final[str] = "range_sp_auto_air_min"
API_RANGE_SP_MIN_COOL_AIR: Final[str] = "range_sp_cool_air_min"
API_RANGE_SP_MIN_DRY_AIR: Final[str] = "range_sp_dry_air_min"
API_RANGE_SP_MIN_EMERHEAT_AIR: Final[str] = "range_sp_emerheat_air_min"
API_RANGE_SP_MIN_HOT_AIR: Final[str] = "range_sp_hot_air_min"
API_RANGE_SP_MIN_STOP_AIR: Final[str] = "range_sp_stop_air_min"
API_RANGE_SP_MIN_VENT_AIR: Final[str] = "range_sp_vent_air_min"
API_REFRESH_TOKEN: Final[str] = "refreshToken"
API_RETURN_TEMP: Final[str] = "return_temp"
API_SETPOINT: Final[str] = "setpoint"
API_SLEEP: Final[str] = "sleep"
API_SP_AIR_AUTO: Final[str] = "setpoint_air_auto"
API_SP_AIR_COOL: Final[str] = "setpoint_air_cool"
API_SP_AIR_DRY: Final[str] = "setpoint_air_dry"
API_SP_AIR_HEAT: Final[str] = "setpoint_air_heat"
API_SP_AIR_STOP: Final[str] = "setpoint_air_stop"
API_SP_AIR_VENT: Final[str] = "setpoint_air_vent"
API_SPEED_CONF: Final[str] = "speed_conf"
API_SPEED_TYPE: Final[str] = "speed_type"
API_SPEED_VALUES: Final[str] = "speed_values"
API_STAT_AP_MAC: Final[str] = "stat_ap_mac"
API_STAT_CHANNEL: Final[str] = "stat_channel"
API_STAT_QUALITY: Final[str] = "stat_quality"
API_STAT_RSSI: Final[str] = "stat_rssi"
API_STAT_SSID: Final[str] = "stat_ssid"
API_STATUS: Final[str] = "status"
API_STEP: Final[str] = "step"
API_SYSTEM_FW: Final[str] = "system_fw"
API_SYSTEM_NUMBER: Final[str] = "system_number"
API_SYSTEM_TYPE: Final[str] = "system_type"
API_TANK_TEMP: Final[str] = "tank_temp"
API_THERMOSTAT_FW: Final[str] = "thermostat_fw"
API_THERMOSTAT_TYPE: Final[str] = "thermostat_type"
API_TT_UNITS: Final[str] = "tt_units"
API_TIMER: Final[str] = "timer"
API_TOKEN: Final[str] = "token"
API_TYPE: Final[str] = "type"
API_TYPE_ALL: Final[str] = "all"
API_TYPE_ADVANCED: Final[str] = "advanced"
API_TYPE_USER: Final[str] = "user"
API_UNITS: Final[str] = "units"
API_URL: Final[str] = f"https://{AIRZONE_SERVER}"
API_USER: Final[str] = "user"
API_USER_ID: Final[str] = "user_id"
API_USER_LOGOUT: Final[str] = "user/logout"
API_USER_MODE_CONF: Final[str] = "usermode_conf"
API_V1: Final[str] = "api/v1"
API_VALUE: Final[str] = "value"
API_WARNINGS: Final[str] = "warnings"
API_WORK_TEMP: Final[str] = "work_temp"
API_WS: Final[str] = "ws"
API_WS_CONNECTED: Final[str] = "ws_connected"
API_WS_FW: Final[str] = "ws_fw"
API_WS_ID: Final[str] = "ws_id"
API_WS_IDS: Final[str] = "ws_ids"
API_WS_TYPE: Final[str] = "ws_type"
API_ZONE_NUMBER: Final[str] = "zone_number"

API_AQ_STATUS: Final[dict[str, int]] = {
    "good": 1,
    "regular": 151,
    "bad": 301,
}
API_AZ_AIDOO: Final[str] = "aidoo"
API_AZ_AIDOO_ACS: Final[str] = "aidoo_acs"
API_AZ_AIDOO_PRO: Final[str] = "aidoo_it"
API_AZ_ACS: Final[str] = "az_acs"
API_AZ_SYSTEM: Final[str] = "az_system"
API_AZ_ZONE: Final[str] = "az_zone"

API_DEFAULT_TEMP_STEP: Final[float] = 0.5
API_DEFAULT_TEMP_STEP_ACS: Final[int] = 1

AZD_ACTION: Final[str] = "action"
AZD_ACTIVE: Final[str] = "active"
AZD_AIDOOS: Final[str] = "aidoos"
AZD_AIR_DEMAND: Final[str] = "air-demand"
AZD_AQ_ACTIVE: Final[str] = "aq-active"
AZD_AQ_INDEX: Final[str] = "aq-index"
AZD_AQ_MODE_CONF: Final[str] = "aq-mode-conf"
AZD_AQ_MODE_VALUES: Final[str] = "aq-mode-values"
AZD_AQ_PM_1: Final[str] = "aq-pm-1"
AZD_AQ_PM_2P5: Final[str] = "aq-pm-2.5"
AZD_AQ_PM_10: Final[str] = "aq-pm-10"
AZD_AQ_PRESENT: Final[str] = "aq-present"
AZD_AQ_STATUS: Final[str] = "aq-status"
AZD_AVAILABLE: Final[str] = "available"
AZD_CONNECTION_DATE: Final[str] = "connection-date"
AZD_DISCONNECTION_DATE: Final[str] = "disconnection-date"
AZD_DOUBLE_SET_POINT: Final[str] = "double-set-point"
AZD_ERRORS: Final[str] = "errors"
AZD_FIRMWARE: Final[str] = "firmware"
AZD_FLOOR_DEMAND: Final[str] = "floor-demand"
AZD_GROUPS: Final[str] = "groups"
AZD_HUMIDITY: Final[str] = "humidity"
AZD_HOT_WATERS: Final[str] = "hot-water"
AZD_ID: Final[str] = "id"
AZD_INDOOR_EXCHANGER_TEMP: Final[str] = "indoor-exchanger-temperature"
AZD_INDOOR_RETURN_TEMP: Final[str] = "indoor-return-temperature"
AZD_INDOOR_WORK_TEMP: Final[str] = "indoor-work-temperature"
AZD_INSTALLATION: Final[str] = "installation"
AZD_INSTALLATIONS: Final[str] = "installations"
AZD_IS_CONNECTED: Final[str] = "is-connected"
AZD_MASTER: Final[str] = "master"
AZD_MODE: Final[str] = "mode"
AZD_MODE_AUTO: Final[str] = "mode-auto"
AZD_MODEL: Final[str] = "model"
AZD_MODES: Final[str] = "modes"
AZD_NAME: Final[str] = "name"
AZD_NUM_DEVICES: Final[str] = "num-devices"
AZD_NUM_GROUPS: Final[str] = "num-groups"
AZD_OPERATION: Final[str] = "operation"
AZD_OPERATIONS: Final[str] = "operations"
AZD_OUTDOOR_DISCHARGE_PRESS: Final[str] = "outdoor-discharge-pressure"
AZD_OUTDOOR_DISCHARGE_TEMP: Final[str] = "outdoor-discharge-temperature"
AZD_OUTDOOR_ELECTRIC_CURRENT: Final[str] = "outdoor-electric-current"
AZD_OUTDOOR_EXCHANGER_TEMP: Final[str] = "outdoor-exchanger-temperature"
AZD_OUTDOOR_TEMP: Final[str] = "outdoor-temperature"
AZD_POWER: Final[str] = "power"
AZD_POWER_MODE: Final[str] = "power-mode"
AZD_PROBLEMS: Final[str] = "problems"
AZD_SPEED: Final[str] = "speed"
AZD_SPEEDS: Final[str] = "speeds"
AZD_SPEED_TYPE: Final[str] = "speed-type"
AZD_SYSTEM: Final[str] = "system"
AZD_SYSTEM_ID: Final[str] = "system-id"
AZD_SYSTEMS: Final[str] = "systems"
AZD_TEMP: Final[str] = "temperature"
AZD_TEMP_STEP: Final[str] = "temperature-step"
AZD_TEMP_SET: Final[str] = "temperature-setpoint"
AZD_TEMP_SET_AUTO_AIR: Final[str] = "temperature-setpoint-auto-air"
AZD_TEMP_SET_COOL_AIR: Final[str] = "temperature-setpoint-cool-air"
AZD_TEMP_SET_DRY_AIR: Final[str] = "temperature-setpoint-dry-air"
AZD_TEMP_SET_HOT_AIR: Final[str] = "temperature-setpoint-hot-air"
AZD_TEMP_SET_STOP_AIR: Final[str] = "temperature-setpoint-stop-air"
AZD_TEMP_SET_VENT_AIR: Final[str] = "temperature-setpoint-vent-air"
AZD_TEMP_SET_MAX: Final[str] = "temperature-setpoint-max"
AZD_TEMP_SET_MAX_AUTO_AIR: Final[str] = "temperature-setpoint-max-auto-air"
AZD_TEMP_SET_MAX_COOL_AIR: Final[str] = "temperature-setpoint-max-cool-air"
AZD_TEMP_SET_MAX_DRY_AIR: Final[str] = "temperature-setpoint-max-dry-air"
AZD_TEMP_SET_MAX_EMERHEAT_AIR: Final[str] = "temperature-setpoint-max-emerheat-air"
AZD_TEMP_SET_MAX_HOT_AIR: Final[str] = "temperature-setpoint-max-hot-air"
AZD_TEMP_SET_MAX_STOP_AIR: Final[str] = "temperature-setpoint-max-stop-air"
AZD_TEMP_SET_MAX_VENT_AIR: Final[str] = "temperature-setpoint-max-vent-air"
AZD_TEMP_SET_MIN_AUTO_AIR: Final[str] = "temperature-setpoint-min-auto-air"
AZD_TEMP_SET_MIN_COOL_AIR: Final[str] = "temperature-setpoint-min-cool-air"
AZD_TEMP_SET_MIN_DRY_AIR: Final[str] = "temperature-setpoint-min-dry-air"
AZD_TEMP_SET_MIN_EMERHEAT_AIR: Final[str] = "temperature-setpoint-min-emerheat-air"
AZD_TEMP_SET_MIN_HOT_AIR: Final[str] = "temperature-setpoint-min-hot-air"
AZD_TEMP_SET_MIN_STOP_AIR: Final[str] = "temperature-setpoint-min-stop-air"
AZD_TEMP_SET_MIN_VENT_AIR: Final[str] = "temperature-setpoint-min-vent-air"
AZD_TEMP_SET_MIN: Final[str] = "temperature-setpoint-min"
AZD_THERMOSTAT_BATTERY: Final[str] = "thermostat-battery"
AZD_THERMOSTAT_COVERAGE: Final[str] = "thermostat-coverage"
AZD_THERMOSTAT_FW: Final[str] = "thermostat-fw"
AZD_THERMOSTAT_MODEL: Final[str] = "thermostat-model"
AZD_TYPE: Final[str] = "type"
AZD_USER_ACCESS: Final[str] = "user-access"
AZD_WARNINGS: Final[str] = "warnings"
AZD_WEBSERVER: Final[str] = "web-server"
AZD_WEBSERVERS: Final[str] = "web-servers"
AZD_WIFI_CHANNEL: Final[str] = "wifi-channel"
AZD_WIFI_MAC: Final[str] = "wifi-mac"
AZD_WIFI_QUALITY: Final[str] = "wifi-quality"
AZD_WIFI_RSSI: Final[str] = "wifi-rssi"
AZD_WIFI_SSID: Final[str] = "wifi-ssid"
AZD_WS_CONNECTED: Final[str] = "ws-connected"
AZD_ZONE: Final[str] = "zone"
AZD_ZONES: Final[str] = "zones"

HEADER_AUTHORIZATION: Final[str] = "Authorization"
HEADER_BEARER: Final[str] = "Bearer"

HTTP_CALL_TIMEOUT: Final[int] = 45
HTTP_MAX_REQUESTS: Final[int] = 4

RAW_DEVICES_CONFIG: Final[str] = "devices-config"
RAW_DEVICES_STATUS: Final[str] = "devices-status"
RAW_INSTALLATIONS: Final[str] = "installations"
RAW_INSTALLATIONS_LIST: Final[str] = "installations-list"
RAW_USER: Final[str] = "user"
RAW_WEBSERVERS: Final[str] = "webservers"

TOKEN_REFRESH_PERIOD: Final[timedelta] = timedelta(hours=12)

WS_ALIVE_PERIOD: Final[timedelta] = timedelta(seconds=45)
WS_AUTH: Final[str] = "auth"
WS_BODY: Final[str] = "body"
WS_CHANGE: Final[str] = "change"
WS_CORR_ID: Final[str] = "corrId"
WS_DEVICE_STATE: Final[str] = "DEVICE_STATE"
WS_DEVICE_STATE_END: Final[str] = "DEVICE_STATE_END"
WS_DEVICES_UPDATES: Final[str] = "DEVICES_UPDATES"
WS_EVENT: Final[str] = "event"
WS_INIT_TIMEOUT: Final[int] = 15
WS_INSTALLATION: Final[str] = "installation"
WS_INSTALLATION_ID: Final[str] = "installationId"
WS_JWT: Final[str] = "jwt"
WS_STATUS: Final[str] = "status"
WS_URL: Final[str] = f"wss://{AIRZONE_SERVER}"
WS_WEBSERVER_UPDATES: Final[str] = "WEBSERVER_UPDATES"
WS_WEBSOCKETS: Final[str] = "websockets"



"""Airzone Cloud API Domestic Hot Water."""

from __future__ import annotations

from typing import Any

from .common import HotWaterOperation, parse_bool, parse_float, parse_int
from .const import (
    API_ACTIVE,
    API_CELSIUS,
    API_DEFAULT_TEMP_STEP_ACS,
    API_POWER,
    API_POWERFUL_MODE,
    API_RANGE_SP_MAX_ACS,
    API_RANGE_SP_MIN_ACS,
    API_SETPOINT,
    API_STEP,
    API_TANK_TEMP,
    API_VALUE,
    AZD_ACTIVE,
    AZD_OPERATION,
    AZD_OPERATIONS,
    AZD_POWER,
    AZD_POWER_MODE,
    AZD_TEMP,
    AZD_TEMP_SET,
    AZD_TEMP_SET_MAX,
    AZD_TEMP_SET_MIN,
    AZD_TEMP_STEP,
)
from .device import Device
from .entity import EntityUpdate, UpdateType


class HotWater(Device):
    """Airzone Cloud Domestic Hot Water device."""

    def __init__(self, inst_id: str, ws_id: str, device_data: dict[str, Any]):
        """Airzone Cloud DHW device init."""
        super().__init__(inst_id, ws_id, device_data)

        self.active: bool | None = None
        self.name: str = "Airzone Cloud DHW"
        self.power: bool | None = None
        self.power_mode: bool | None = None
        self.temp_set_max: int | None = None
        self.temp_set_min: int | None = None
        self.temp_set: int | None = None
        self.temp: float | None = None
        self.temp_step: int | None = None

    def data(self) -> dict[str, Any]:
        """Return DHW device data."""
        data = super().data()

        data[AZD_ACTIVE] = self.get_active()
        data[AZD_OPERATION] = self.get_operation()
        data[AZD_OPERATIONS] = self.get_operations()
        data[AZD_POWER] = self.get_power()
        data[AZD_TEMP] = self.get_temperature()
        data[AZD_TEMP_SET] = self.get_temp_set()
        data[AZD_TEMP_SET_MAX] = self.get_temp_set_max()
        data[AZD_TEMP_SET_MIN] = self.get_temp_set_min()
        data[AZD_TEMP_STEP] = self.get_temp_step()

        power_mode = self.get_power_mode()
        if power_mode is not None:
            data[AZD_POWER_MODE] = power_mode

        return data

    def get_active(self) -> bool | None:
        """Return DHW device active status."""
        return self.active

    def get_operation(self) -> HotWaterOperation:
        """Return DHW current operation."""
        if self.get_power():
            if self.get_power_mode():
                return HotWaterOperation.Powerful
            return HotWaterOperation.On
        return HotWaterOperation.Off

    def get_operations(self) -> list[HotWaterOperation]:
        """Return DHW operation list."""
        operations = [
            HotWaterOperation.Off,
            HotWaterOperation.On,
        ]
        if self.get_power_mode() is not None:
            operations += [HotWaterOperation.Powerful]
        return operations

    def get_power(self) -> bool | None:
        """Return DHW device power."""
        return self.power

    def get_power_mode(self) -> bool | None:
        """Return DHW device power_mode."""
        return self.power_mode

    def get_temperature(self) -> float | None:
        """Return DHW device temperature."""
        if self.temp is not None:
            return round(self.temp, 1)
        return None

    def get_temp_set(self) -> int | None:
        """Return DHW device temperature setpoint."""
        return self.temp_set

    def get_temp_set_max(self) -> int | None:
        """Return DHW device max setpoint."""
        return self.temp_set_max

    def get_temp_set_min(self) -> int | None:
        """Return DHW device min setpoint."""
        return self.temp_set_min

    def get_temp_step(self) -> int | None:
        """Return DHW device temperature step."""
        if self.temp_step is not None:
            return self.temp_step
        return API_DEFAULT_TEMP_STEP_ACS

    def set_power(self, power: bool) -> None:
        """Set DHW power."""
        self.power = power

    def set_powerful_mode(self, powerful_mode: bool) -> None:
        """Set DHW powerful mode."""
        self.power_mode = powerful_mode

    def set_setpoint(self, setpoint: int) -> None:
        """Set DHW setpoint."""
        if self.temp_set is not None:
            self.temp_set = setpoint

    def set_param(self, param: str, data: dict[str, Any]) -> None:
        """Update DHW parameter from API request."""
        if param == API_POWER:
            self.set_power(data[API_VALUE])
        elif param == API_POWERFUL_MODE:
            self.set_powerful_mode(data[API_VALUE])
        elif param == API_SETPOINT:
            self.set_setpoint(data[API_VALUE])

    def update_data(self, update: EntityUpdate) -> None:
        """Update DHW device data."""
        super().update_data(update)

        data = update.get_data()

        if API_ACTIVE in data:
            active = parse_bool(data.get(API_ACTIVE))
            if active is not None:
                self.active = active
            else:
                # API sends active as null instead of False
                self.active = False
        else:
            if update.get_type() != UpdateType.WS_PARTIAL:
                self.active = None

        power = parse_bool(data.get(API_POWER))
        if power is not None:
            self.power = power

        powerful_mode = parse_bool(data.get(API_POWERFUL_MODE))
        if powerful_mode is not None:
            self.power_mode = powerful_mode

        range_sp_max_acs = parse_int(
            data.get(API_RANGE_SP_MAX_ACS, {}).get(API_CELSIUS)
        )
        if range_sp_max_acs is not None:
            self.temp_set_max = range_sp_max_acs

        range_sp_min_acs = parse_int(
            data.get(API_RANGE_SP_MIN_ACS, {}).get(API_CELSIUS)
        )
        if range_sp_min_acs is not None:
            self.temp_set_min = range_sp_min_acs

        setpoint = parse_int(data.get(API_SETPOINT, {}).get(API_CELSIUS))
        if setpoint is not None:
            self.temp_set = setpoint

        step = parse_int(data.get(API_STEP, {}).get(API_CELSIUS))
        if step is not None:
            self.temp_step = step

        tank_temp = parse_float(data.get(API_TANK_TEMP, {}).get(API_CELSIUS))
        if tank_temp is not None:
            self.temp = tank_temp



"""Airzone Cloud API Token."""

from __future__ import annotations

from datetime import datetime
from typing import Any
import urllib.parse

from .const import (
    API_AUTH_REFRESH_TOKEN,
    API_REFRESH_TOKEN,
    API_TOKEN,
    API_V1,
    HEADER_AUTHORIZATION,
    HEADER_BEARER,
    TOKEN_REFRESH_PERIOD,
    WS_JWT,
)
from .exceptions import LoginError, TokenRefreshError


class AirzoneCloudToken:
    """Airzone Cloud Token."""

    def __init__(self) -> None:
        """Airzone Cloud Token init."""
        self.refresh_time: datetime | None = None
        self.refresh_token: str | None = None
        self.token: str | None = None

    def check_refresh(self) -> bool:
        """Check if Airzone Cloud Token needs refreshing."""
        return (
            self.refresh_time is not None
            and (datetime.now() - self.refresh_time) >= TOKEN_REFRESH_PERIOD
        )

    def clear(self) -> None:
        """Clear Airzone Cloud Token."""
        self.refresh_time = None
        self.refresh_token = None
        self.token = None

    def headers(self) -> dict[str, Any]:
        """Airzone Cloud Token headers."""
        _headers: dict[str, Any] = {}

        if self.token is not None:
            _headers[HEADER_AUTHORIZATION] = f"{HEADER_BEARER} {self.token}"

        return _headers

    def is_valid(self) -> bool:
        """Check if Airzone Cloud Token is valid."""
        return (
            self.refresh_time is not None
            and self.refresh_token is not None
            and self.token is not None
        )

    def jwt(self) -> dict[str, Any]:
        """Airzone Cloud WebSockets headers."""
        _jwt: dict[str, Any] = {}

        if self.token is not None:
            _jwt[WS_JWT] = self.token

        return _jwt

    def update(self, resp: dict[str, Any], refresh: bool) -> None:
        """Update Airzone Cloud Token."""
        refresh_token = resp.get(API_REFRESH_TOKEN)
        token = resp.get(API_TOKEN)

        if refresh_token is None or token is None:
            if refresh:
                raise TokenRefreshError("Invalid API response")
            raise LoginError("Invalid API response")

        self.refresh_time = datetime.now()
        self.refresh_token = refresh_token
        self.token = token

    def url_refresh(self) -> str:
        """Airzone Cloud Token refresh URL."""
        refresh_token = urllib.parse.quote(self.refresh_token or "")
        return f"{API_V1}/{API_AUTH_REFRESH_TOKEN}/{refresh_token}"



"""Airzone Cloud API."""

from __future__ import annotations

import asyncio
from asyncio import Event, Lock, Task
from datetime import datetime
from json import JSONDecodeError, loads as json_loads
import logging
from typing import TYPE_CHECKING, Any
import urllib.parse

from aiohttp import ClientSession, ClientWebSocketResponse, WSMessage, WSMsgType

from .const import (
    API_DEVICE_ID,
    API_V1,
    API_WS_ID,
    WS_ALIVE_PERIOD,
    WS_AUTH,
    WS_BODY,
    WS_CORR_ID,
    WS_DEVICE_STATE,
    WS_DEVICE_STATE_END,
    WS_DEVICES_UPDATES,
    WS_EVENT,
    WS_INSTALLATION,
    WS_INSTALLATION_ID,
    WS_URL,
    WS_WEBSERVER_UPDATES,
    WS_WEBSOCKETS,
)
from .device import Device
from .entity import EntityUpdate, UpdateType
from .installation import Installation
from .token import AirzoneCloudToken

if TYPE_CHECKING:
    from .cloudapi import AirzoneCloudApi

_LOGGER = logging.getLogger(__name__)


class AirzoneCloudIWS:
    """Airzone Cloud Installation WebSockets."""

    def __init__(
        self,
        cloudapi: AirzoneCloudApi,
        installation: Installation,
    ):
        """Airzone Cloud WebSockets init."""
        self.alive_dt: datetime | None = None
        self.cloudapi: AirzoneCloudApi = cloudapi
        self.device_data_lock = Lock()
        self.device_data: dict[str, Any] = {}
        self.inst_id: str = installation.get_id()
        self.session: ClientSession = cloudapi.session or ClientSession()
        self.state_end: Event = Event()
        self.task: Task[None] | None = None
        self.token: AirzoneCloudToken = cloudapi.token

    async def _connect(self) -> None:
        """WebSockets connection."""
        params = {
            WS_INSTALLATION_ID: self.inst_id,
        }
        inst_params = urllib.parse.urlencode(params)
        url = f"{WS_URL}/{API_V1}/{WS_WEBSOCKETS}/{WS_INSTALLATION}?{inst_params}"

        await self.state_init()

        async with self.session.ws_connect(
            url,
            headers=self.token.headers(),
            autoclose=False,
            autoping=False,
        ) as ws:
            async for msg in ws:
                await self.handler(ws, msg)

    def connect(self) -> bool:
        """WebSockets task creation."""
        if self.task is not None:
            return self.is_connected()

        self.state_end.clear()
        self.task = asyncio.ensure_future(self._connect())

        return True

    def disconnect(self) -> bool:
        """WebSockets task deletion."""
        self.state_end.clear()

        task = self.task
        if task is None:
            return True

        res = task.cancel()
        self.task = None

        return res

    def reconnect(self) -> bool:
        """WebSockets reconnect."""
        _LOGGER.warning("WS[%s]: reconnecting...", self.inst_id)
        self.disconnect()
        return self.connect()

    def get_device_data(self, device: Device) -> dict[str, Any] | None:
        """Return WebSockets device data."""
        return self.device_data.get(device.get_id())

    async def handler_auth(
        self, ws: ClientWebSocketResponse, data: dict[str, Any]
    ) -> None:
        """WebSockets AUTH handler."""
        corr_id = data.get(WS_CORR_ID)
        if corr_id is not None:
            auth = {
                WS_CORR_ID: corr_id,
                WS_BODY: self.token.jwt(),
            }
            _LOGGER.debug("WS[%s]: AUTH[%s]", self.inst_id, corr_id)
            await ws.send_json(auth)
        else:
            _LOGGER.error("WS[%s]: AUTH error -> %s", self.inst_id, data)

    async def handler_close(self, ws: ClientWebSocketResponse) -> None:
        """WebSockets CLOSE handler."""
        _LOGGER.debug("WS[%s]: CLOSE", self.inst_id)
        await ws.close()

    async def handler_device_state(self, data: dict[str, Any]) -> None:
        """WebSockets DEVICE_STATE handler."""
        body: dict[str, Any] = data.get(WS_BODY, {})
        update = EntityUpdate(UpdateType.WS_FULL, body)
        dev_id: str | None = body.get(API_DEVICE_ID)

        _LOGGER.debug("WS[%s]: DEVICE_STATE[%s]", self.inst_id, dev_id)

        device = self.cloudapi.get_device_id(dev_id)
        if device is not None:
            async with self.device_data_lock:
                self.device_data[device.get_id()] = body

            await device.update(update)

    def handler_device_state_end(self, data: dict[str, Any]) -> None:
        """WebSockets DEVICE_STATE_END handler."""
        body: str | None = data.get(WS_BODY)

        if body == self.inst_id:
            _LOGGER.debug("WS[%s]: DEVICE_STATE_END", self.inst_id)
            self.state_end.set()
        else:
            _LOGGER.error("WS[%s]: DEVICE_STATE_END mismatch (%s)", self.inst_id, body)

    async def handler_devices_update(self, data: dict[str, Any]) -> None:
        """WebSockets DEVICES_UPDATES handler."""
        body: dict[str, Any] = data.get(WS_BODY, {})
        update = EntityUpdate(UpdateType.WS_PARTIAL, body)
        dev_id: str | None = body.get(API_DEVICE_ID)

        _LOGGER.debug("WS[%s]: DEVICES_UPDATES[%s]", self.inst_id, dev_id)

        device = self.cloudapi.get_device_id(dev_id)
        if device is not None:
            await device.update(update)

    async def handler_error(self, msg: WSMessage) -> None:
        """WebSockets ERROR handler."""
        _LOGGER.error("WS[%s]: ERROR -> %s", self.inst_id, msg)

    async def handler_ping(self, ws: ClientWebSocketResponse) -> None:
        """WebSockets PING handler."""
        _LOGGER.debug("WS[%s]: PING (%s)", self.inst_id, datetime.now())
        await ws.pong()

    async def handler_text(
        self, ws: ClientWebSocketResponse, data: dict[str, Any]
    ) -> None:
        """WebSockets TEXT handler."""
        event: str = data.get(WS_EVENT, "")
        if event == WS_AUTH:
            await self.handler_auth(ws, data)
        elif event == WS_DEVICE_STATE:
            await self.handler_device_state(data)
        elif event == WS_DEVICE_STATE_END:
            self.handler_device_state_end(data)
        elif event.startswith(WS_DEVICES_UPDATES):
            await self.handler_devices_update(data)
            self.cloudapi.update_callback()
        elif event.startswith(WS_WEBSERVER_UPDATES):
            await self.handler_webserver_updates(data)
            self.cloudapi.update_callback()
        else:
            _LOGGER.warning("WS[%s]: EVENT[%s] -> %s", self.inst_id, event, data)

    async def handler_webserver_updates(self, data: dict[str, Any]) -> None:
        """WebSockets WEBSERVER_UPDATES handler."""
        body: dict[str, Any] = data.get(WS_BODY, {})
        update = EntityUpdate(UpdateType.WS_PARTIAL, body)
        ws_id: str | None = body.get(API_WS_ID)

        _LOGGER.debug("WS[%s]: WEBSERVER_UPDATES[%s]", self.inst_id, ws_id)

        webserver = self.cloudapi.get_webserver_id(ws_id)
        if webserver is not None:
            await webserver.update(update)

    async def handler(self, ws: ClientWebSocketResponse, msg: WSMessage) -> None:
        """WebSockets message handler."""
        if msg.type == WSMsgType.TEXT:
            json_data = None
            try:
                json_data = json_loads(msg.data)
            except (JSONDecodeError, TypeError) as err:
                _LOGGER.error(err)

            if json_data is not None:
                self.set_alive()
                await self.handler_text(ws, json_data)
        elif msg.type == WSMsgType.PING:
            self.set_alive()
            await self.handler_ping(ws)
        elif msg.type == WSMsgType.CLOSE:
            await self.handler_close(ws)
        elif msg.type == WSMsgType.ERROR:
            await self.handler_error(msg)
        else:
            _LOGGER.warning("Unknown WS msg: %s", msg)

    def is_alive(self) -> bool:
        """WebSockets connection alive."""
        return (
            self.alive_dt is not None
            and (datetime.now() - self.alive_dt) <= WS_ALIVE_PERIOD
        )

    def is_connected(self) -> bool:
        """WebSockets connection status."""
        task = self.task

        if task is None:
            return False

        return not task.done()

    def set_alive(self) -> None:
        """WebSockets alive status update."""
        self.alive_dt = datetime.now()

    async def state_init(self) -> None:
        """WebSockets state init."""
        _LOGGER.debug("WS[%s]: DEVICE_STATE_INIT", self.inst_id)

        async with self.device_data_lock:
            self.device_data.clear()

        self.alive_dt = None
        self.state_end.clear()



"""Airzone Cloud API Device."""

from __future__ import annotations

from abc import abstractmethod
import logging
from typing import Any

from .common import OperationMode, parse_bool, parse_int, parse_str
from .const import (
    API_AQ_PM_1,
    API_AQ_PM_2P5,
    API_AQ_PM_10,
    API_AQ_PRESENT,
    API_AQ_QUALITY,
    API_AQ_STATUS,
    API_CONFIG,
    API_DEVICE_ID,
    API_ERRORS,
    API_IS_CONNECTED,
    API_META,
    API_MODE,
    API_MODE_AVAIL,
    API_SYSTEM_NUMBER,
    API_WARNINGS,
    API_WS_CONNECTED,
    AZD_AQ_INDEX,
    AZD_AQ_PM_1,
    AZD_AQ_PM_2P5,
    AZD_AQ_PM_10,
    AZD_AQ_PRESENT,
    AZD_AQ_STATUS,
    AZD_AVAILABLE,
    AZD_ERRORS,
    AZD_ID,
    AZD_INSTALLATION,
    AZD_IS_CONNECTED,
    AZD_MODE,
    AZD_MODE_AUTO,
    AZD_MODES,
    AZD_NAME,
    AZD_PROBLEMS,
    AZD_WARNINGS,
    AZD_WEBSERVER,
    AZD_WS_CONNECTED,
)
from .entity import Entity, EntityUpdate

_LOGGER = logging.getLogger(__name__)


class Device(Entity):
    """Airzone Cloud Device."""

    def __init__(self, inst_id: str, ws_id: str, device_data: dict[str, Any]):
        """Airzone Cloud Device init."""
        super().__init__()

        self.auto_mode: OperationMode | None = None
        self.aq_pm_1: int | None = None
        self.aq_pm_2p5: int | None = None
        self.aq_pm_10: int | None = None
        self.aq_present: bool | None = None
        self.aq_status: str | None = None
        self.errors: list[str] = []
        self.id = str(device_data[API_DEVICE_ID])
        self.installation_id = inst_id
        self.mode: OperationMode | None = None
        self.modes: list[OperationMode] = []
        self.name: str = "Device"
        self.warnings: list[str] = []
        self.webserver_id = ws_id
        self.ws_connected: bool = True

        is_connected = parse_bool(device_data.get(API_IS_CONNECTED))
        if is_connected is not None:
            self.is_connected = is_connected
        else:
            self.is_connected = True

    def sub_data(self, device_data: dict[str, Any]) -> dict[str, Any]:
        """Get Device sub data."""
        if API_META in device_data:
            meta: dict[str, Any] = device_data.get(API_META, {})
            if API_SYSTEM_NUMBER in meta:
                return meta

        if API_CONFIG in device_data:
            config: dict[str, Any] = device_data.get(API_CONFIG, {})
            if API_SYSTEM_NUMBER in config:
                return config

        return device_data

    def data(self) -> dict[str, Any]:
        """Return Device data."""
        data: dict[str, Any] = {
            AZD_AVAILABLE: self.get_available(),
            AZD_ID: self.get_id(),
            AZD_INSTALLATION: self.get_installation(),
            AZD_IS_CONNECTED: self.get_is_connected(),
            AZD_NAME: self.get_name(),
            AZD_PROBLEMS: self.get_problems(),
            AZD_WEBSERVER: self.get_webserver(),
            AZD_WS_CONNECTED: self.get_ws_connected(),
        }

        aq_index = self.get_aq_index()
        if aq_index is not None:
            data[AZD_AQ_INDEX] = aq_index

        aq_pm_1 = self.get_aq_pm_1()
        if aq_pm_1 is not None:
            data[AZD_AQ_PM_1] = aq_pm_1

        aq_pm_2p5 = self.get_aq_pm_2p5()
        if aq_pm_2p5 is not None:
            data[AZD_AQ_PM_2P5] = aq_pm_2p5

        aq_pm_10 = self.get_aq_pm_10()
        if aq_pm_10 is not None:
            data[AZD_AQ_PM_10] = aq_pm_10

        aq_present = self.get_aq_present()
        if aq_present is not None:
            data[AZD_AQ_PRESENT] = aq_present

        aq_status = self.get_aq_status()
        if aq_status is not None:
            data[AZD_AQ_STATUS] = aq_status

        errors = self.get_errors()
        if len(errors) > 0:
            data[AZD_ERRORS] = errors

        mode = self.get_mode()
        if mode is not None:
            data[AZD_MODE] = mode

        mode_auto = self.get_mode_auto()
        if mode_auto is not None:
            data[AZD_MODE_AUTO] = mode_auto

        modes = self.get_modes()
        if modes is not None:
            data[AZD_MODES] = modes

        warnings = self.get_warnings()
        if len(warnings) > 0:
            data[AZD_WARNINGS] = warnings

        return data

    def get_aq_index(self) -> int | None:
        """Return HVAC device Air Quality index."""
        if self.aq_status is not None:
            for key, value in API_AQ_STATUS.items():
                if self.aq_status == key:
                    return value
        return None

    def get_aq_pm_1(self) -> int | None:
        """Return HVAC device Air Quality PM 1."""
        return self.aq_pm_1

    def get_aq_pm_2p5(self) -> int | None:
        """Return HVAC device Air Quality PM 2.5."""
        return self.aq_pm_2p5

    def get_aq_pm_10(self) -> int | None:
        """Return HVAC device Air Quality PM 10."""
        return self.aq_pm_10

    def get_aq_present(self) -> bool | None:
        """Return HVAC device Air Quality present."""
        return self.aq_present

    def get_aq_status(self) -> str | None:
        """Return HVAC device Air Quality status."""
        return self.aq_status

    def get_available(self) -> bool:
        """Return availability status."""
        return self.is_connected and self.ws_connected

    def get_errors(self) -> list[str]:
        """Return Device errors."""
        return self.errors

    def get_id(self) -> str:
        """Return Device ID."""
        return self.id

    def get_installation(self) -> str:
        """Return Installation ID."""
        return self.installation_id

    def get_is_connected(self) -> bool:
        """Return Device connection status."""
        return self.is_connected

    def get_mode(self) -> OperationMode | None:
        """Return Device mode."""
        return self.mode

    def get_mode_auto(self) -> OperationMode | None:
        """Return current auto mode."""
        return self.auto_mode

    def get_modes(self) -> list[OperationMode] | None:
        """Return Device modes."""
        if len(self.modes) > 0:
            return self.modes
        return None

    def get_name(self) -> str:
        """Return Device name."""
        return self.name

    def get_problems(self) -> bool:
        """Return Device problems."""
        return bool(self.errors) or bool(self.warnings)

    def get_warnings(self) -> list[str]:
        """Return Device warnings."""
        return self.warnings

    def get_webserver(self) -> str:
        """Return WebServer ID."""
        return self.webserver_id

    def get_ws_connected(self) -> bool:
        """Return WebServer connection status."""
        return self.ws_connected

    def set_mode(self, mode: int | OperationMode) -> None:
        """Set device operation mode."""
        _mode = OperationMode(mode)
        if _mode in self.modes:
            self.mode = _mode
        else:
            _LOGGER.error("%s: mode %s not in %s", self.id, _mode, self.modes)

    @abstractmethod
    def set_param(self, param: str, data: dict[str, Any]) -> None:
        """Update device parameter from API request."""

    def update_data(self, update: EntityUpdate) -> None:
        """Update Device data."""
        data = update.get_data()

        is_connected = parse_bool(data.get(API_IS_CONNECTED))
        if is_connected is not None:
            self.is_connected = is_connected
        ws_connected = parse_bool(data.get(API_WS_CONNECTED))
        if ws_connected is not None:
            self.ws_connected = ws_connected

        aq_pm_1 = parse_int(data.get(API_AQ_PM_1))
        if aq_pm_1 is not None:
            self.aq_pm_1 = aq_pm_1

        aq_pm_2p5 = parse_int(data.get(API_AQ_PM_2P5))
        if aq_pm_2p5 is not None:
            self.aq_pm_2p5 = aq_pm_2p5

        aq_pm_10 = parse_int(data.get(API_AQ_PM_10))
        if aq_pm_10 is not None:
            self.aq_pm_10 = aq_pm_10

        aq_present = parse_bool(data.get(API_AQ_PRESENT))
        if aq_present is not None:
            self.aq_present = aq_present

        aq_status = parse_str(data.get(API_AQ_QUALITY))
        if aq_status is not None:
            self.aq_status = aq_status

        errors = data.get(API_ERRORS)
        if errors is not None:
            self.errors = []
            for error in errors:
                self.errors += [error]

        mode = data.get(API_MODE)
        if mode is not None:
            self.mode = OperationMode(mode)
        mode_avail = data.get(API_MODE_AVAIL)
        if mode_avail is not None and len(mode_avail) > 0:
            modes = []
            for mode in mode_avail:
                modes += [OperationMode(mode)]
            self.modes = modes

        warnings = data.get(API_WARNINGS)
        if warnings is not None:
            self.warnings = []
            for warning in warnings:
                self.warnings += [warning]
